@@ -39,8 +39,8 @@
 #' robu_CR2
 #' coef_test(robu_fit, vcov = robu_CR2, test = c("Satterthwaite", "saddlepoint"))
 #' 
-#' Wald_test(robu_fit, constraints = c(2,4), vcov = robu_CR2)
-#' Wald_test(robu_fit, constraints = 2:5, vcov = robu_CR2)
+#' Wald_test(robu_fit, constraints = constrain_zero(c(2,4)), vcov = robu_CR2)
+#' Wald_test(robu_fit, constraints = constrain_zero(2:5), vcov = robu_CR2)
 #' 
 
 
@@ -57,10 +57,13 @@ vcovCR.robu <- function(obj, cluster, type, target, inverse_var, form = "sandwic
 # coefficients
 #-----------------------------------------------
 
+#' @export
+
 coef_CS.robu <- function(obj) {
   beta <- as.vector(obj$b.r)
   labs <- obj$reg_table$labels
-  names(beta) <- levels(labs)[labs]
+  if (is.factor(labs)) labs <- levels(labs)[labs]
+  names(beta) <- labs
   beta
 }
 
@@ -68,9 +71,18 @@ coef_CS.robu <- function(obj) {
 # residuals
 #-----------------------------------------------
 
+#' @export
+
 residuals_CS.robu <- function(obj) {
   ord <- order(order(obj$study_orig_id))
-  obj$data.full$e.r[ord]
+  resid <- obj$data.full$e.r[ord]
+    
+  if (obj$user_weighting) {
+    pos_wts <- obj$data.full$userweights[ord] > 0
+    if (!all(pos_wts)) resid <- resid[pos_wts]
+  } 
+  
+  return(resid)
 }
 
 
@@ -78,21 +90,33 @@ residuals_CS.robu <- function(obj) {
 # Model matrix
 #-----------------------------------------------
 
+#' @export
+
 model_matrix.robu <- function(obj) {
   ord <- order(order(obj$study_orig_id))
-  obj$Xreg[ord,]
+  model_matrix <- obj$Xreg[ord,,drop=FALSE]
+  
+  if (obj$user_weighting) {
+    pos_wts <- obj$data.full$userweights[ord] > 0
+    if (!all(pos_wts)) model_matrix <- model_matrix[pos_wts,,drop=FALSE]
+  }
+  
+  return(model_matrix)
 }
 
 #-------------------------------------
 # Get (model-based) working variance matrix 
 #-------------------------------------
 
+#' @export
+
 targetVariance.robu <- function(obj, cluster) {
   ord <- order(order(obj$study_orig_id))
   if (obj$user_weighting) {
-    V <- obj$data.full$avg.var.eff.size[ord]
+    pos_wts <- obj$data.full$userweights[ord] > 0
+    V <- obj$data.full$avg.var.eff.size[ord][pos_wts]
   } else {
-    V <- 1 / obj$data.full$r.weights[ord]
+    V <- mean(obj$data.full$r.weights) / obj$data.full$r.weights[ord]
   }
   matrix_list(V, cluster, "both")
 }
@@ -101,20 +125,40 @@ targetVariance.robu <- function(obj, cluster) {
 # Get weighting matrix
 #-------------------------------------
 
+#' @export
+
+weights.robu <- function(object, ...) {
+  ord <- order(order(object$study_orig_id))
+  if (object$user_weighting) { 
+    object$data.full$userweights[ord]
+  } else{
+    NULL
+  }
+}
+
+#' @export
+
 weightMatrix.robu <- function(obj, cluster) {
   ord <- order(order(obj$study_orig_id))
   if (obj$user_weighting) { 
     W <- obj$data.full$userweights[ord]
+    W <- W[W > 0]
   } else{
     W <- obj$data.full$r.weights[ord]
   }
-  matrix_list(W, cluster, "both")
+  w_scale <- mean(W)
+  W <- W / w_scale
+  W_list <- matrix_list(W, cluster, "both")
+  attr(W_list, "w_scale") <- w_scale
+  W_list
 }
 
 #---------------------------------------
 # Get bread matrix and scaling constant
 #---------------------------------------
 
+#' @export
+
 bread.robu <- function(x, ...) {
   if (x$user_weighting) { 
     W <- x$data.full$userweights
@@ -124,6 +168,8 @@ bread.robu <- function(x, ...) {
   x$N * chol2inv(chol(crossprod(x$Xreg, W * x$Xreg)))
 }
 
+#' @export
+
 v_scale.robu <- function(obj) {
   obj$N
 }
