@@ -12,7 +12,8 @@ egsingle <- droplevels(subset(egsingle, schoolid %in% school_subset))
 
 obj_A1 <- lme(math ~ year * size + female + black + hispanic,
               random = list(~ year | schoolid, ~ 1 | childid),
-              data = egsingle)
+              data = egsingle,
+              control = lmeControl(tolerance = 1e-4, opt = "optim"))
 obj_A2 <- update(obj_A1, weights = varIdent(form = ~ 1 | female))
 obj_A3 <- update(obj_A1, correlation = corExp(form = ~ year))
 obj_A4 <- update(obj_A2, correlation = corExp(form = ~ year))
@@ -33,7 +34,7 @@ test_that("bread works", {
 test_that("vcovCR options work for CR2", {
   skip_on_cran()
   
-  expect_identical(vcovCR(obj_A1, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A1"]])
+  expect_equal(vcovCR(obj_A1, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A1"]])
   expect_equal(vcovCR(obj_A1, type = "CR2", inverse_var = TRUE), CR2_mats[["A1"]])
   expect_false(identical(vcovCR(obj_A1, type = "CR2", inverse_var = FALSE), CR2_mats[["A1"]]))
   target <- targetVariance(obj_A1)
@@ -41,7 +42,7 @@ test_that("vcovCR options work for CR2", {
   attr(CR2_mats[["A1"]], "inverse_var") <- FALSE
   expect_equal(vcovCR(obj_A1, type = "CR2", target = target, inverse_var = FALSE), CR2_mats[["A1"]])
 
-  expect_identical(vcovCR(obj_A2, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A2"]])
+  expect_equal(vcovCR(obj_A2, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A2"]])
   expect_equal(vcovCR(obj_A2, type = "CR2", inverse_var = TRUE), CR2_mats[["A2"]])
   expect_false(identical(vcovCR(obj_A2, type = "CR2", inverse_var = FALSE), CR2_mats[["A2"]]))
   target <- targetVariance(obj_A2)
@@ -49,7 +50,7 @@ test_that("vcovCR options work for CR2", {
   attr(CR2_mats[["A2"]], "inverse_var") <- FALSE
   expect_equal(vcovCR(obj_A2, type = "CR2", target = target, inverse_var = FALSE), CR2_mats[["A2"]])
 
-  expect_identical(vcovCR(obj_A3, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A3"]])
+  expect_equal(vcovCR(obj_A3, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A3"]])
   expect_equal(vcovCR(obj_A3, type = "CR2", inverse_var = TRUE), CR2_mats[["A3"]])
   expect_false(identical(vcovCR(obj_A3, type = "CR2", inverse_var = FALSE), CR2_mats[["A3"]]))
   target <- targetVariance(obj_A3)
@@ -57,7 +58,7 @@ test_that("vcovCR options work for CR2", {
   attr(CR2_mats[["A3"]], "inverse_var") <- FALSE
   expect_equal(vcovCR(obj_A3, type = "CR2", target = target, inverse_var = FALSE), CR2_mats[["A3"]])
   
-  expect_identical(vcovCR(obj_A4, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A4"]])
+  expect_equal(vcovCR(obj_A4, cluster = egsingle$schoolid, type = "CR2"), CR2_mats[["A4"]])
   expect_equal(vcovCR(obj_A4, type = "CR2", inverse_var = TRUE), CR2_mats[["A4"]])
   expect_false(identical(vcovCR(obj_A4, type = "CR2", inverse_var = FALSE), CR2_mats[["A4"]]))
   target <- targetVariance(obj_A4)
@@ -72,32 +73,32 @@ test_that("vcovCR options work for CR4", {
   skip("Not worrying about CR4 for now.")
   CR4_mats <- lapply(objects, vcovCR, type = "CR4")
   
-  expect_identical(vcovCR(obj_A1, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A1"]])
-  expect_identical(vcovCR(obj_A1, type = "CR4", inverse_var = TRUE), CR4_mats[["A1"]])
+  expect_equal(vcovCR(obj_A1, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A1"]])
+  expect_equal(vcovCR(obj_A1, type = "CR4", inverse_var = TRUE), CR4_mats[["A1"]])
   expect_false(identical(vcovCR(obj_A1, type = "CR4", inverse_var = FALSE), CR4_mats[["A1"]]))
   target <- targetVariance(obj_A1)
   expect_equal(vcovCR(obj_A1, type = "CR4", target = target, inverse_var = TRUE), CR4_mats[["A1"]])
   attr(CR4_mats[["A1"]], "inverse_var") <- FALSE
   expect_equal(vcovCR(obj_A1, type = "CR4", target = target, inverse_var = FALSE), CR4_mats[["A1"]])
   
-  expect_identical(vcovCR(obj_A2, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A2"]])
-  expect_identical(vcovCR(obj_A2, type = "CR4", inverse_var = TRUE), CR4_mats[["A2"]])
+  expect_equal(vcovCR(obj_A2, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A2"]])
+  expect_equal(vcovCR(obj_A2, type = "CR4", inverse_var = TRUE), CR4_mats[["A2"]])
   expect_false(identical(vcovCR(obj_A2, type = "CR4", inverse_var = FALSE), CR4_mats[["A2"]]))
   target <- targetVariance(obj_A2)
   expect_equal(vcovCR(obj_A2, type = "CR4", target = target, inverse_var = TRUE), CR4_mats[["A2"]])
   attr(CR4_mats[["A2"]], "inverse_var") <- FALSE
   expect_equal(vcovCR(obj_A2, type = "CR4", target = target, inverse_var = FALSE), CR4_mats[["A2"]])
   
-  expect_identical(vcovCR(obj_A3, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A3"]])
-  expect_identical(vcovCR(obj_A3, type = "CR4", inverse_var = TRUE), CR4_mats[["A3"]])
+  expect_equal(vcovCR(obj_A3, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A3"]])
+  expect_equal(vcovCR(obj_A3, type = "CR4", inverse_var = TRUE), CR4_mats[["A3"]])
   expect_false(identical(vcovCR(obj_A3, type = "CR4", inverse_var = FALSE), CR4_mats[["A3"]]))
   target <- targetVariance(obj_A3)
   expect_equal(vcovCR(obj_A3, type = "CR4", target = target, inverse_var = TRUE), CR4_mats[["A3"]])
   attr(CR4_mats[["A3"]], "inverse_var") <- FALSE
   expect_equal(vcovCR(obj_A3, type = "CR4", target = target, inverse_var = FALSE), CR4_mats[["A3"]])
   
-  expect_identical(vcovCR(obj_A4, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A4"]])
-  expect_identical(vcovCR(obj_A4, type = "CR4", inverse_var = TRUE), CR4_mats[["A4"]])
+  expect_equal(vcovCR(obj_A4, cluster = egsingle$schoolid, type = "CR4"), CR4_mats[["A4"]])
+  expect_equal(vcovCR(obj_A4, type = "CR4", inverse_var = TRUE), CR4_mats[["A4"]])
   expect_false(identical(vcovCR(obj_A4, type = "CR4", inverse_var = FALSE), CR4_mats[["A4"]]))
   target <- targetVariance(obj_A4)
   expect_equal(vcovCR(obj_A4, type = "CR4", target = target, inverse_var = TRUE), CR4_mats[["A4"]])
@@ -133,20 +134,37 @@ test_that("clubSandwich works with dropped observations", {
   obj_dropped <- update(obj_A4, data = dat_miss, na.action = na.omit)
   obj_complete <- update(obj_A4, data = dat_miss, subset = !is.na(math))
 
-  obj <- obj_dropped
-  cluster <- nlme::getGroups(obj, level = 1)
-  target <- NULL
-  inverse_var <- is.null(target)
-  type <- "CR2"
-  form <- "sandwich"
+  # obj <- obj_dropped
+  # cluster <- nlme::getGroups(obj, level = 1)
+  # target <- NULL
+  # inverse_var <- TRUE
+  # type <- "CR2"
+  # form <- "sandwich"
+  # 
+  # full_grps <- get_cor_grouping(obj)
+  # R_list <- nlme::corMatrix(obj$modelStruct$corStruct)
+  # levels <- names(R_list)
+  # grps <- get_cor_grouping(obj, levels = names(R_list))
+  # 
+  # V_list <- build_var_cor_mats(obj)
+  # V_grps <- attr(V_list, "groups")
+  # ZDZ_list <- build_RE_mats(obj)
+  # ZDZ_grps <- attr(ZDZ_list, "groups")
+  # 
+  # V_dim <- sapply(V_list, nrow)
+  # identical(names(V_dim), names(table(V_grps)))
+  # data.frame(dim = V_dim, grps = table(V_grps))
+  # table(V_dim == table(V_grps))
+  # dat_miss$x <- NA
+  # dat_miss$x[!is.na(dat_miss$math)] <- V_grps
   
   CR_drop <- lapply(CR_types, function(x) vcovCR(obj_dropped, type = x))
   CR_complete <- lapply(CR_types, function(x) vcovCR(obj_complete, type = x))
-  expect_identical(CR_drop, CR_complete)
+  expect_equal(CR_drop, CR_complete)
 
   test_drop <- lapply(CR_drop, function(x) coef_test(obj_dropped, vcov = x, test = "All", p_values = FALSE))
   test_complete <- lapply(CR_complete, function(x) coef_test(obj_complete, vcov = x, test = "All", p_values = FALSE))
-  expect_identical(test_drop, test_complete)
+  expect_equal(test_drop, test_complete)
 })
 
 
